package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.ButtonDimmerDimRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.GenericDeviceSaveReguest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import java.security.Principal;
import java.util.List;
import java.util.Set;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/buttonDimmer")
public class ButtonDimmerController
        extends InputDeviceConnectionController<ButtonDimmer, DimmableLight> {
    private ButtonDimmerRepository buttonDimmerRepository;
    private DimmableLightRepository dimmableLightRepository;

    @Autowired
    protected ButtonDimmerController(
            ButtonDimmerRepository inputRepository, DimmableLightRepository outputRepository) {
        super(
                inputRepository,
                outputRepository,
                DimmableLight.BUTTON_DIMMER_DIMMABLE_LIGHT_CONNECTOR);
        this.buttonDimmerRepository = inputRepository;
        this.dimmableLightRepository = outputRepository;
    }

    @GetMapping
    public List<ButtonDimmer> findAll() {
        return toList(buttonDimmerRepository.findAll());
    }

    @GetMapping("/{id}")
    public ButtonDimmer findById(@PathVariable("id") long id) throws NotFoundException {
        return buttonDimmerRepository.findById(id).orElseThrow(NotFoundException::new);
    }

    @PostMapping
    public ButtonDimmer create(@Valid @RequestBody final GenericDeviceSaveReguest bd) {
        ButtonDimmer newBD = new ButtonDimmer();
        newBD.setName(bd.getName());
        newBD.setRoomId(bd.getRoomId());

        return buttonDimmerRepository.save(newBD);
    }

    @PutMapping("/dim")
    public Set<DimmableLight> dim(
            @Valid @RequestBody final ButtonDimmerDimRequest bd, final Principal principal)
            throws NotFoundException {
        final ButtonDimmer buttonDimmer =
                buttonDimmerRepository
                        .findByIdAndUsername(bd.getId(), principal.getName())
                        .orElseThrow(NotFoundException::new);

        switch (bd.getDimType()) {
            case UP:
                buttonDimmer.increaseIntensity();
                break;
            case DOWN:
                buttonDimmer.decreaseIntensity();
                break;
        }

        dimmableLightRepository.saveAll(buttonDimmer.getOutputs());

        return buttonDimmer.getOutputs();
    }

    @PostMapping("/{id}/lights")
    public Set<? extends OutputDevice> addLight(
            @PathVariable("id") long inputId, @RequestParam("lightId") Long lightId)
            throws NotFoundException {
        return addOutput(inputId, lightId);
    }

    @DeleteMapping("/{id}/lights")
    public Set<? extends OutputDevice> removeLight(
            @PathVariable("id") long inputId, @RequestParam("lightId") Long lightId)
            throws NotFoundException {
        return removeOutput(inputId, lightId);
    }

    @DeleteMapping("/{id}")
    public void delete(@PathVariable("id") long id) {
        buttonDimmerRepository.deleteById(id);
    }
}
