package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.DimmableLightSaveRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.DimmableLight;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.DimmableLightRepository;
import java.security.Principal;
import java.util.List;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/dimmableLight")
public class DimmableLightController {

    @Autowired private DimmableLightRepository dimmableLightService;

    @GetMapping
    public List<DimmableLight> findAll() {
        return toList(dimmableLightService.findAll());
    }

    @GetMapping("/{id}")
    public DimmableLight findById(@PathVariable("id") long id) throws NotFoundException {
        return dimmableLightService.findById(id).orElseThrow(NotFoundException::new);
    }

    private DimmableLight save(DimmableLight initial, DimmableLightSaveRequest dl) {
        initial.setIntensity(dl.getIntensity());
        initial.setName(dl.getName());
        initial.setRoomId(dl.getRoomId());

        return dimmableLightService.save(initial);
    }

    @PostMapping
    public DimmableLight create(@Valid @RequestBody DimmableLightSaveRequest dl) {
        return save(new DimmableLight(), dl);
    }

    @PutMapping
    public DimmableLight update(
            @Valid @RequestBody DimmableLightSaveRequest sp, final Principal principal)
            throws NotFoundException {
        return save(
                dimmableLightService
                        .findByIdAndUsername(sp.getId(), principal.getName())
                        .orElseThrow(NotFoundException::new),
                sp);
    }

    @DeleteMapping("/{id}")
    public void delete(@PathVariable("id") long id) {
        dimmableLightService.deleteById(id);
    }
}
