package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.GenericDeviceSaveReguest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.KnobDimmerDimRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import java.security.Principal;
import java.util.List;
import java.util.Set;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/knobDimmer")
public class KnobDimmerController
        extends InputDeviceConnectionController<KnobDimmer, DimmableLight> {

    @Autowired private KnobDimmerRepository knobDimmerRepository;
    @Autowired private DimmableLightRepository dimmableLightRepository;

    @Autowired
    protected KnobDimmerController(
            KnobDimmerRepository inputRepository, DimmableLightRepository outputRepository) {
        super(
                inputRepository,
                outputRepository,
                DimmableLight.KNOB_DIMMER_DIMMABLE_LIGHT_CONNECTOR);
        this.knobDimmerRepository = inputRepository;
        this.dimmableLightRepository = outputRepository;
    }

    @GetMapping
    public List<KnobDimmer> findAll() {
        return toList(knobDimmerRepository.findAll());
    }

    @GetMapping("/{id}")
    public KnobDimmer findById(@PathVariable("id") long id) throws NotFoundException {
        return knobDimmerRepository.findById(id).orElseThrow(NotFoundException::new);
    }

    @PostMapping
    public KnobDimmer create(@Valid @RequestBody GenericDeviceSaveReguest kd) {
        KnobDimmer newKD = new KnobDimmer();
        newKD.setName(kd.getName());
        newKD.setRoomId(kd.getRoomId());

        return knobDimmerRepository.save(newKD);
    }

    @PutMapping("/dimTo")
    public Set<DimmableLight> dimTo(
            @Valid @RequestBody final KnobDimmerDimRequest bd, final Principal principal)
            throws NotFoundException {
        final KnobDimmer dimmer =
                knobDimmerRepository
                        .findByIdAndUsername(bd.getId(), principal.getName())
                        .orElseThrow(NotFoundException::new);

        dimmer.setLightIntensity(bd.getIntensity());
        dimmableLightRepository.saveAll(dimmer.getOutputs());

        return dimmer.getOutputs();
    }

    @PostMapping("/{id}/lights")
    public Set<? extends OutputDevice> addLight(
            @PathVariable("id") long inputId, @RequestParam("lightId") Long lightId)
            throws NotFoundException {
        return addOutput(inputId, lightId);
    }

    @DeleteMapping("/{id}/lights")
    public Set<? extends OutputDevice> removeLight(
            @PathVariable("id") long inputId, @RequestParam("lightId") Long lightId)
            throws NotFoundException {
        return removeOutput(inputId, lightId);
    }

    @DeleteMapping("/{id}")
    public void delete(@PathVariable("id") long id) {
        knobDimmerRepository.deleteById(id);
    }
}
