package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.GenericDeviceSaveReguest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.MotionSensor;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.MotionSensorRepository;
import ch.usi.inf.sa4.sanmarinoes.smarthut.socket.SensorSocketEndpoint;
import java.security.Principal;
import java.util.List;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/motionSensor")
public class MotionSensorController {

    @Autowired private MotionSensorRepository motionSensorService;

    @Autowired private SensorSocketEndpoint sensorSocketEndpoint;

    @GetMapping
    public List<MotionSensor> findAll() {
        return toList(motionSensorService.findAll());
    }

    @GetMapping("/{id}")
    public MotionSensor findById(@PathVariable("id") long id) throws NotFoundException {
        return motionSensorService.findById(id).orElseThrow(NotFoundException::new);
    }

    @PostMapping
    public MotionSensor create(@Valid @RequestBody GenericDeviceSaveReguest ms) {
        MotionSensor newMS = new MotionSensor();
        newMS.setName(ms.getName());
        newMS.setRoomId(ms.getRoomId());

        return motionSensorService.save(newMS);
    }

    /**
     * Updates detection status of given motion sensor and propagates update throgh socket
     *
     * @param sensor the motion sensor to update
     * @param detected the new detection status
     * @return the updated motion sensor
     */
    public MotionSensor updateDetectionFromMotionSensor(MotionSensor sensor, boolean detected) {
        sensor.setDetected(detected);
        final MotionSensor toReturn = motionSensorService.save(sensor);

        sensorSocketEndpoint.broadcast(sensor, motionSensorService.findUser(sensor.getId()));

        return toReturn;
    }

    @PutMapping("/{id}/detect")
    public MotionSensor updateDetection(
            @PathVariable("id") Long sensorId,
            @RequestParam("detected") boolean detected,
            final Principal principal)
            throws NotFoundException {

        return updateDetectionFromMotionSensor(
                motionSensorService
                        .findByIdAndUsername(sensorId, principal.getName())
                        .orElseThrow(NotFoundException::new),
                detected);
    }

    @DeleteMapping("/{id}")
    public void delete(@PathVariable("id") long id) {
        motionSensorService.deleteById(id);
    }
}
