package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.RegularLightSaveRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.RegularLight;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.RegularLightRepository;
import java.security.Principal;
import java.util.List;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@EnableAutoConfiguration
@RequestMapping("/regularLight")
public class RegularLightController {

    @Autowired private RegularLightRepository regularLightService;

    @GetMapping
    public List<RegularLight> findAll() {
        return toList(regularLightService.findAll());
    }

    @GetMapping("/{id}")
    public RegularLight findById(@PathVariable("id") long id) throws NotFoundException {
        return regularLightService.findById(id).orElseThrow(NotFoundException::new);
    }

    private RegularLight save(RegularLight newRL, RegularLightSaveRequest rl) {
        newRL.setName(rl.getName());
        newRL.setRoomId(rl.getRoomId());
        newRL.setOn(rl.isOn());

        return regularLightService.save(newRL);
    }

    @PostMapping
    public RegularLight create(@Valid @RequestBody RegularLightSaveRequest rl) {
        return save(new RegularLight(), rl);
    }

    @PutMapping
    public RegularLight update(
            @Valid @RequestBody RegularLightSaveRequest rl, final Principal principal)
            throws NotFoundException {
        return save(
                regularLightService
                        .findByIdAndUsername(rl.getId(), principal.getName())
                        .orElseThrow(NotFoundException::new),
                rl);
    }

    @DeleteMapping("/{id}")
    public void delete(@PathVariable("id") long id) {
        regularLightService.deleteById(id);
    }
}
