package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.RoomSaveRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import java.security.Principal;
import java.util.*;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.*;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/room")
public class RoomController {

    @Autowired private RoomRepository roomRepository;

    @Autowired private UserRepository userRepository;

    @Autowired private DeviceRepository<Device> deviceRepository;

    @Autowired private SwitchRepository switchRepository;

    @Autowired private ButtonDimmerRepository buttonDimmerRepository;

    @Autowired private KnobDimmerRepository knobDimmerRepository;

    @GetMapping
    public List<Room> findAll() {
        return toList(roomRepository.findAll());
    }

    @GetMapping("/{id}")
    public @ResponseBody Room findById(@PathVariable("id") long id) throws NotFoundException {
        return roomRepository.findById(id).orElseThrow(NotFoundException::new);
    }

    @PostMapping
    public @ResponseBody Room create(
            @Valid @RequestBody RoomSaveRequest r, final Principal principal) {

        final String username = principal.getName();
        final Long userId = userRepository.findByUsername(username).getId();
        final String img = r.getImage();
        final Room.Icon icon = r.getIcon();

        final Room newRoom = new Room();
        newRoom.setUserId(userId);
        newRoom.setName(r.getName());
        newRoom.setImage(img);
        newRoom.setIcon(icon);

        return roomRepository.save(newRoom);
    }

    @PutMapping("/{id}")
    public @ResponseBody Room update(
            @PathVariable("id") long id, @RequestBody RoomSaveRequest r, final Principal principal)
            throws NotFoundException {
        final Room newRoom =
                roomRepository
                        .findByIdAndUsername(id, principal.getName())
                        .orElseThrow(NotFoundException::new);
        final String img = r.getImage();
        final Room.Icon icon = r.getIcon();

        if (r.getName() != null) {
            newRoom.setName(r.getName());
        }

        if ("".equals(img)) {
            newRoom.setImage(null);
        } else if (img != null) {
            newRoom.setImage(img);
        }

        if (icon != null) {
            newRoom.setIcon(icon);
        }

        return roomRepository.save(newRoom);
    }

    @DeleteMapping("/{id}")
    public void deleteById(@PathVariable("id") long id) {
        switchRepository.deleteAllByRoomId(id);
        knobDimmerRepository.deleteAllByRoomId(id);
        buttonDimmerRepository.deleteAllByRoomId(id);
        roomRepository.deleteById(id);
    }

    /**
     * Returns a List<Device> of all Devices that are present in a given room (identified by its
     * id).
     */
    @GetMapping(path = "/{roomId}/devices")
    public List<Device> getDevices(@PathVariable("roomId") long roomid) {
        return deviceRepository.findByRoomId(roomid);
    }
}
