package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.SensorSaveRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import ch.usi.inf.sa4.sanmarinoes.smarthut.socket.SensorSocketEndpoint;
import java.math.BigDecimal;
import java.security.Principal;
import java.util.*;
import java.util.List;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.*;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/sensor")
public class SensorController {

    @Autowired private SensorRepository sensorRepository;

    @Autowired private SensorSocketEndpoint sensorSocketEndpoint;

    @GetMapping
    public List<Sensor> findAll() {
        return toList(sensorRepository.findAll());
    }

    @GetMapping("/{id}")
    public Sensor findById(@PathVariable("id") long id) throws NotFoundException {
        return sensorRepository.findById(id).orElseThrow(NotFoundException::new);
    }

    @PostMapping
    public Sensor create(@Valid @RequestBody SensorSaveRequest s) {
        Sensor newSensor = new Sensor();
        newSensor.setSensor(s.getSensor());
        newSensor.setName(s.getName());
        newSensor.setRoomId(s.getRoomId());
        newSensor.setValue(s.getValue());

        return sensorRepository.save(newSensor);
    }

    /**
     * Updates the sensor with new measurement and propagates update through websocket
     *
     * @param sensor the sensor to update
     * @param value the new measurement
     * @return the updated sensor
     */
    public Sensor updateValueFromSensor(Sensor sensor, BigDecimal value) {
        sensor.setValue(value);
        final Sensor toReturn = sensorRepository.save(sensor);

        sensorSocketEndpoint.broadcast(sensor, sensorRepository.findUser(sensor.getId()));

        return toReturn;
    }

    @PutMapping("/{id}/value")
    public Sensor updateValue(
            @PathVariable("id") Long sensorId,
            @RequestParam("value") BigDecimal value,
            final Principal principal)
            throws NotFoundException {
        return updateValueFromSensor(
                sensorRepository
                        .findByIdAndUsername(sensorId, principal.getName())
                        .orElseThrow(NotFoundException::new),
                value);
    }

    @DeleteMapping("/{id}")
    public void deleteById(@PathVariable("id") long id) {
        sensorRepository.deleteById(id);
    }
}
