package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.SmartPlugSaveRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import java.security.Principal;
import java.util.*;
import java.util.List;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.*;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/smartPlug")
public class SmartPlugController {

    @Autowired private SmartPlugRepository smartPlugRepository;

    @GetMapping
    public List<SmartPlug> findAll() {
        return toList(smartPlugRepository.findAll());
    }

    @GetMapping("/{id}")
    public SmartPlug findById(@PathVariable("id") long id) throws NotFoundException {
        return smartPlugRepository.findById(id).orElseThrow(NotFoundException::new);
    }

    private SmartPlug save(SmartPlug newSP, SmartPlugSaveRequest sp) {
        newSP.setOn(sp.isOn());
        newSP.setId(sp.getId());
        newSP.setName(sp.getName());
        newSP.setRoomId(sp.getRoomId());

        return smartPlugRepository.save(newSP);
    }

    @PostMapping
    public SmartPlug create(@Valid @RequestBody SmartPlugSaveRequest sp) {
        return save(new SmartPlug(), sp);
    }

    @PutMapping
    public SmartPlug update(@Valid @RequestBody SmartPlugSaveRequest sp, final Principal principal)
            throws NotFoundException {
        return save(
                smartPlugRepository
                        .findByIdAndUsername(sp.getId(), principal.getName())
                        .orElseThrow(NotFoundException::new),
                sp);
    }

    @DeleteMapping("/{id}/meter")
    public SmartPlug resetMeter(@PathVariable("id") long id, final Principal principal)
            throws NotFoundException {
        final SmartPlug s =
                smartPlugRepository
                        .findByIdAndUsername(id, principal.getName())
                        .orElseThrow(NotFoundException::new);

        s.resetTotalConsumption();
        return smartPlugRepository.save(s);
    }

    @DeleteMapping("/{id}")
    public void deleteById(@PathVariable("id") long id) {
        smartPlugRepository.deleteById(id);
    }
}
