package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static ch.usi.inf.sa4.sanmarinoes.smarthut.utils.Utils.toList;

import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.GenericDeviceSaveReguest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.dto.SwitchOperationRequest;
import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import java.security.Principal;
import java.util.*;
import java.util.List;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.*;
import org.springframework.web.bind.annotation.*;

@RestController
@EnableAutoConfiguration
@RequestMapping("/switch")
public class SwitchController extends InputDeviceConnectionController<Switch, Switchable> {

    private SwitchRepository switchRepository;
    private SwitchableRepository<Switchable> switchableRepository;

    /**
     * Contstructs the controller by requiring essential object for the controller implementation
     *
     * @param inputRepository the input device repository
     * @param outputRepository the output device repository
     */
    @Autowired
    protected SwitchController(
            SwitchRepository inputRepository, SwitchableRepository<Switchable> outputRepository) {
        super(inputRepository, outputRepository, Switchable.SWITCH_SWITCHABLE_CONNECTOR);
        this.switchRepository = inputRepository;
        this.switchableRepository = outputRepository;
    }

    @GetMapping
    public List<Switch> findAll() {
        return toList(switchRepository.findAll());
    }

    @GetMapping("/{id}")
    public Switch findById(@PathVariable("id") long id) throws NotFoundException {
        return switchRepository.findById(id).orElseThrow(NotFoundException::new);
    }

    @PostMapping
    public Switch create(@Valid @RequestBody GenericDeviceSaveReguest s) {
        Switch newSwitch = new Switch();
        newSwitch.setName(s.getName());
        newSwitch.setRoomId(s.getRoomId());

        return switchRepository.save(newSwitch);
    }

    @PutMapping("/operate")
    public Set<Switchable> operate(
            @Valid @RequestBody final SwitchOperationRequest sr, final Principal principal)
            throws NotFoundException {
        final Switch s =
                switchRepository
                        .findByIdAndUsername(sr.getId(), principal.getName())
                        .orElseThrow(NotFoundException::new);

        switch (sr.getType()) {
            case ON:
                s.setOn(true);
                break;
            case OFF:
                s.setOn(false);
                break;
            case TOGGLE:
                s.toggle();
                break;
        }

        switchableRepository.saveAll(s.getOutputs());

        return s.getOutputs();
    }

    @PostMapping("/{id}/lights")
    public Set<? extends OutputDevice> addSwitchable(
            @PathVariable("id") long inputId, @RequestParam("switchableId") Long switchableId)
            throws NotFoundException {
        return addOutput(inputId, switchableId);
    }

    @DeleteMapping("/{id}/lights")
    public Set<? extends OutputDevice> removeSwitchable(
            @PathVariable("id") long inputId, @RequestParam("switchableId") Long switchableId)
            throws NotFoundException {
        return removeOutput(inputId, switchableId);
    }

    @DeleteMapping("/{id}")
    public void deleteById(@PathVariable("id") long id) {
        switchRepository.deleteById(id);
    }
}
