package ch.usi.inf.sa4.sanmarinoes.smarthut.dto;

import ch.usi.inf.sa4.sanmarinoes.smarthut.models.Sensor;
import com.google.gson.annotations.SerializedName;
import java.math.BigDecimal;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.validation.constraints.NotNull;

public class SensorSaveRequest {

    /** Type of sensor, i.e. of the thing the sensor measures. */
    public enum SensorType {
        /** A sensor that measures temperature in degrees celsius */
        @SerializedName("TEMPERATURE")
        TEMPERATURE,

        /** A sensor that measures relative humidity in percentage points */
        @SerializedName("HUMIDITY")
        HUMIDITY,

        /** A sensor that measures light in degrees */
        @SerializedName("LIGHT")
        LIGHT
    }

    /** The type of this sensor */
    @NotNull
    @Enumerated(value = EnumType.STRING)
    private Sensor.SensorType sensor;

    @NotNull private BigDecimal value;

    /**
     * The room this device belongs in, as a foreign key id. To use when updating and inserting from
     * a REST call.
     */
    @NotNull private Long roomId;

    /** The name of the device as assigned by the user (e.g. 'Master bedroom light') */
    @NotNull private String name;

    public void setRoomId(Long roomId) {
        this.roomId = roomId;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Long getRoomId() {
        return roomId;
    }

    public String getName() {
        return name;
    }

    public Sensor.SensorType getSensor() {
        return sensor;
    }

    public void setSensor(Sensor.SensorType sensor) {
        this.sensor = sensor;
    }

    public BigDecimal getValue() {
        return value;
    }

    public void setValue(BigDecimal value) {
        this.value = value;
    }
}
