package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import ch.usi.inf.sa4.sanmarinoes.smarthut.config.GsonExclude;
import com.google.gson.annotations.SerializedName;
import io.swagger.annotations.ApiModelProperty;
import javax.persistence.*;
import javax.validation.constraints.NotNull;

/** Generic abstraction for a smart home device */
@Entity
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
public abstract class Device {

    /** Ways a device can behave in the automation flow. For now only input/output */
    public enum FlowType {
        @SerializedName("INPUT")
        INPUT,

        @SerializedName("OUTPUT")
        OUTPUT
    }

    /** Device identifier */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", updatable = false, nullable = false, unique = true)
    @ApiModelProperty(hidden = true)
    private long id;

    @ManyToOne
    @JoinColumn(name = "room_id", updatable = false, insertable = false)
    @GsonExclude
    private Room room;

    /**
     * The room this device belongs in, as a foreign key id. To use when updating and inserting from
     * a REST call.
     */
    @Column(name = "room_id", nullable = false)
    @NotNull
    private Long roomId;

    /** The name of the device as assigned by the user (e.g. 'Master bedroom light') */
    @NotNull
    @Column(nullable = false)
    private String name;

    /**
     * The name for the category of this particular device (e.g 'dimmer'). Not stored in the
     * database but set thanks to constructors
     */
    @Transient private final String kind;

    /**
     * The way this device behaves in the automation flow. Not stored in the database but set thanks
     * to constructors
     */
    @Transient private final FlowType flowType;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Long getRoomId() {
        return roomId;
    }

    public void setRoomId(Long roomId) {
        this.roomId = roomId;
    }

    public Device(String kind, FlowType flowType) {
        this.kind = kind;
        this.flowType = flowType;
    }
}
