package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import ch.usi.inf.sa4.sanmarinoes.smarthut.config.GsonExclude;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/** Represent a dimmable light */
@Entity
public class DimmableLight extends Switchable {

    public static final Connector<ButtonDimmer, DimmableLight>
            BUTTON_DIMMER_DIMMABLE_LIGHT_CONNECTOR =
                    Connector.basic(ButtonDimmer::getOutputs, DimmableLight::setDimmerId);

    public static final Connector<KnobDimmer, DimmableLight> KNOB_DIMMER_DIMMABLE_LIGHT_CONNECTOR =
            Connector.basic(KnobDimmer::getOutputs, DimmableLight::setDimmerId);

    public DimmableLight() {
        super("dimmableLight");
    }

    @ManyToOne
    @GsonExclude
    @JoinColumn(name = "dimmer_id", updatable = false, insertable = false)
    private Dimmer dimmer;

    @Column(name = "dimmer_id")
    private Long dimmerId;

    /** The light intensity value. Goes from 0 (off) to 100 (on) */
    @NotNull
    @Column(nullable = false)
    @Min(0)
    @Max(100)
    private Integer intensity = 0;

    @NotNull
    @Column(nullable = false)
    private Integer oldIntensity = 100;

    public Integer getIntensity() {
        return intensity;
    }

    /**
     * Sets the intensity to a certain level. Out of bound values are corrected to the respective
     * extremums. An intensity level of 0 turns the light off, but keeps the old intensity level
     * stored.
     *
     * @param intensity the intensity level (may be out of bounds)
     */
    public void setIntensity(Integer intensity) {
        if (intensity <= 0) {
            this.intensity = 0;
        } else if (intensity > 100) {
            this.intensity = 100;
            this.oldIntensity = 100;
        } else {
            this.intensity = intensity;
            this.oldIntensity = intensity;
        }
    }

    @Override
    public boolean isOn() {
        return intensity != 0;
    }

    @Override
    public void setOn(boolean on) {
        intensity = on ? oldIntensity : 0;
    }

    public void setDimmerId(Long dimmerId) {
        this.dimmerId = dimmerId;
        super.setSwitchId(null);
    }

    @Override
    public void setSwitchId(Long switchId) {
        super.setSwitchId(switchId);
        this.dimmerId = null;
    }
}
