package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import java.util.HashSet;
import java.util.Set;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.OneToMany;
import javax.persistence.PreRemove;

/** Represents a generic dimmer input device */
@Entity
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
public abstract class Dimmer extends InputDevice {
    public Dimmer(String kind) {
        super(kind);
    }

    @OneToMany(mappedBy = "dimmer")
    private Set<DimmableLight> lights = new HashSet<>();

    /**
     * Get the lights connected to this dimmer
     *
     * @return duh
     */
    @Override
    public Set<DimmableLight> getOutputs() {
        return this.lights;
    }

    /** Add a light to be controller by this dimmer */
    public void addDimmableLight(DimmableLight dimmableLight) {
        lights.add(dimmableLight);
    }

    @PreRemove
    private void removeLightsFromDimmer() {
        for (DimmableLight dl : getOutputs()) {
            dl.setDimmerId(null);
        }
    }
}
