package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import java.util.HashSet;
import java.util.Set;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.PreRemove;

/** A switch input device */
@Entity
public class Switch extends InputDevice {

    @OneToMany(mappedBy = "switchDevice")
    private Set<Switchable> switchables = new HashSet<>();

    /** The state of this switch */
    @Column(nullable = false, name = "switch_on")
    private boolean on;

    public Switch() {
        super("switch");
    }

    /**
     * Setter method for this Switch
     *
     * @param state The state to be set
     */
    public void setOn(boolean state) {
        on = state;

        for (final Switchable s : switchables) {
            s.setOn(on);
        }
    }

    /** Toggle between on and off state */
    public void toggle() {
        setOn(!isOn());
    }

    /**
     * Getter method for this Switch
     *
     * @return This Switch on state
     */
    public boolean isOn() {
        return on;
    }

    public Set<Switchable> getOutputs() {
        return switchables;
    }

    @PreRemove
    public void removeSwitchable() {
        for (Switchable s : getOutputs()) {
            s.setSwitchId(null);
        }
    }
}
