package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import ch.usi.inf.sa4.sanmarinoes.smarthut.config.GsonExclude;
import javax.persistence.*;

/** A device that can be turned either on or off */
@Entity
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
public abstract class Switchable extends OutputDevice {

    public static final Connector<Switch, Switchable> SWITCH_SWITCHABLE_CONNECTOR =
            Connector.basic(Switch::getOutputs, Switchable::setSwitchId);

    @ManyToOne
    @GsonExclude
    @JoinColumn(name = "switch_id", updatable = false, insertable = false)
    private Switch switchDevice;

    @Column(name = "switch_id")
    private Long switchId;

    protected Switchable(String kind) {
        super(kind);
    }

    /**
     * Returns whether the device is on (true) or not (false)
     *
     * @return whether the device is on (true) or not (false)
     */
    public abstract boolean isOn();

    /**
     * Sets the on status of the device
     *
     * @param on the new on status: true for on, false for off
     */
    public abstract void setOn(boolean on);

    public Long getSwitchId() {
        return switchId;
    }

    public void setSwitchId(Long switchId) {
        this.switchId = switchId;
    }
}
