package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import ch.usi.inf.sa4.sanmarinoes.smarthut.config.GsonExclude;
import io.swagger.annotations.ApiModelProperty;
import java.util.Objects;
import javax.persistence.*;

/** A user of the Smarthut application */
@Entity(name = "smarthutuser")
public class User {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", updatable = false, nullable = false, unique = true)
    @ApiModelProperty(hidden = true)
    private Long id;

    /** The full name of the user */
    @Column(nullable = false)
    private String name;

    /** The full username of the user */
    @Column(nullable = false, unique = true)
    private String username;

    /** A properly salted way to store the password */
    @Column(nullable = false)
    @GsonExclude
    private String password;

    /**
     * The user's email (validated according to criteria used in <code>&gt;input type="email"&lt;>
     * </code>, technically not RFC 5322 compliant
     */
    @Column(nullable = false, unique = true)
    private String email;

    @Column(nullable = false)
    @GsonExclude
    private Boolean isEnabled = false;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public Boolean getEnabled() {
        return isEnabled;
    }

    public void setEnabled(Boolean enabled) {
        isEnabled = enabled;
    }

    @Override
    public String toString() {
        return "User{"
                + "id="
                + id
                + ", name='"
                + name
                + '\''
                + ", username='"
                + username
                + '\''
                + ", password='"
                + password
                + '\''
                + ", email='"
                + email
                + '\''
                + ", isEnabled="
                + isEnabled
                + '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        User user = (User) o;
        return id.equals(user.id)
                && name.equals(user.name)
                && username.equals(user.username)
                && password.equals(user.password)
                && email.equals(user.email)
                && isEnabled.equals(user.isEnabled);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, username, password, email, isEnabled);
    }
}
