package ch.usi.inf.sa4.sanmarinoes.smarthut.scheduled;

import ch.usi.inf.sa4.sanmarinoes.smarthut.controller.MotionSensorController;
import ch.usi.inf.sa4.sanmarinoes.smarthut.controller.SensorController;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import ch.usi.inf.sa4.sanmarinoes.smarthut.socket.SensorSocketEndpoint;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.StreamSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * Generates fake sensor (and motion sensor) and smart plug consumption updates as required by
 * milestone one
 */
@Component
public class UpdateTasks {

    @Autowired private SensorRepository sensorRepository;

    @Autowired private MotionSensorRepository motionSensorRepository;

    @Autowired private SmartPlugRepository smartPlugRepository;

    @Autowired private SensorController sensorController;

    @Autowired private MotionSensorController motionSensorController;

    @Autowired private SensorSocketEndpoint sensorSocketEndpoint;

    /** Generates fake sensor updates every two seconds with a +/- 1.25% error */
    @Scheduled(fixedRate = 2000)
    public void sensorFakeUpdate() {
        StreamSupport.stream(sensorRepository.findAll().spliterator(), true)
                .forEach(
                        sensor ->
                                sensorController.updateValueFromSensor(
                                        sensor,
                                        Sensor.TYPICAL_VALUES
                                                .get(sensor.getSensor())
                                                .multiply(
                                                        new BigDecimal(
                                                                0.9875 + Math.random() / 40))));
    }

    /**
     * Generate fake motion detections in all motion detectors every 20 seconds for 2 seconds at
     * most
     */
    @Scheduled(fixedDelay = 20000)
    public void motionSensorFakeUpdate() {
        StreamSupport.stream(motionSensorRepository.findAll().spliterator(), true)
                .forEach(
                        sensor -> {
                            motionSensorController.updateDetectionFromMotionSensor(sensor, true);
                            CompletableFuture.delayedExecutor(
                                            (long) (Math.random() * 2000), TimeUnit.MILLISECONDS)
                                    .execute(
                                            () ->
                                                    motionSensorController
                                                            .updateDetectionFromMotionSensor(
                                                                    sensor, false));
                        });
    }

    /** Updates power consumption of all activated smart plugs every second */
    @Scheduled(fixedDelay = 1000)
    public void smartPlugConsumptionFakeUpdate() {
        smartPlugRepository.updateTotalConsumption(SmartPlug.AVERAGE_CONSUMPTION_KW);
        final Collection<SmartPlug> c = smartPlugRepository.findByOn(true);
        c.forEach(s -> sensorSocketEndpoint.broadcast(s, sensorRepository.findUser(s.getId())));
    }
}
